@@ -6,7 +6,8 @@ from ase.constraints import FixAtoms
 import matplotlib.pyplot as plt
 from math import ceil
 from .molecule import packmol
-from .dft.siesta import siesta_md 
+from .dft.siesta import siesta_opt 
+from .dft.qe import qeopt # qemd,
 from .dft.SinglePointEnergy import SinglePointEnergies
 from .md.gulp import nvt as gulp_nvt
 from .data.mdtodata import MDtoData
@@ -52,8 +53,8 @@ def get_zpe(p,atom_name):
 class LearningMachine(object):
   ''' recursive learning machine constructed by I-ReaxFF and siesta '''
   def __init__(self,initConfig='poscar.gen',
-               direcs=None,ncpu=40,
-               maxiter=1000,
+               dataset=None,ncpu=40,
+               maxiter=100,
                dt_mlmd=0.1, dt_aimd=0.3,dft_step=5,
                batch=50,max_batch=100,
                step=20000,md_step=20,MaxMDstep=500,colFrame=10,
@@ -63,7 +64,7 @@ class LearningMachine(object):
                T=300,Tmax=1000,
                label=None,
                convergence=0.01,nconvergence=3,
-               accCriteria=0.9,lossCriteria=1000.0,lossTole=1.0,accTole=0.005,
+               accCriteria=0.9,lossCriteria=10.0,lossTole=1000.0,accTole=0.05,
                accMax=0.9,accInc=1.0001,mdInc=1.2,resetAcc=1,
                learnWay=5,FreePairs=None,beta=None,
                FreeAtoms=None,FirstAtom=None,
@@ -82,7 +83,7 @@ class LearningMachine(object):
                lambda_bd=10000.0,lambda_reg=0.0001,regularize=True,
                lambda_me=0.1,learning_rate=0.0001,
                writelib=5000,ffield='ffield.json',
-               xcf='VDW',xca='DRSLL',basistype='DZP'):
+               dft='siesta',kpts=(1,1,1),xcf='VDW',xca='DRSLL',basistype='DZP'):
       ''' max_batch: max number of batch 
               batch: batch size
       '''
@@ -106,9 +107,11 @@ class LearningMachine(object):
       self.ncpu           = ncpu
       self.step           = step
       self.batch          = batch
-      self.direcs         = direcs
+      self.dataset        = dataset
       self.maxiter        = maxiter
-      self.max_batch      = max_batch   # max number in direcs to train
+      self.max_batch      = max_batch   # max number in dataset to train
+      self.dft            = dft
+      self.kpts           = kpts
       self.xcf            = xcf
       self.xca            = xca
       self.basistype      = basistype
@@ -215,8 +218,8 @@ class LearningMachine(object):
       cwd      = getcwd()
       gen      = 'poscar.gen'
 
-      if self.direcs is None:
-         self.direcs = {}
+      if self.dataset is None:
+         self.dataset = {}
       if not exists(self.aidir):
          mkdir(self.aidir)
 
@@ -259,6 +262,8 @@ class LearningMachine(object):
              continue
           if isfile('md.traj'): 
              atoms = read('md.traj',index=-1)
+             consistance = self.check_atoms(atoms)
+             assert consistance,'-  The species or cell parameter in md.traj is not consistant with initial configuration!'
              # print('-  atomic structure from MD trajectories.')
           else:
              print('-  cannot find MD trajectory, use learnWay=1 in the first iter.')
@@ -304,23 +309,22 @@ class LearningMachine(object):
 
           # self.get_fixatom()
           # atoms.set_constraint(self.fixatoms)
-
-          atoms.write('poscar.gen')                                    # for aimd
+          # atoms.write('poscar.gen')                                    # for aimd
 
           if learnWay==1: 
              dft_step = self.dft_step
           else:
              dft_step = int(mdsteps/self.colFrame)+1
 
-          e_aimd,eml_,dEmax_,d2Emax_,LabelDataLog = self.aimd(cwd,run_dir,dft_step,gen,learnWay)
+          e_aimd,eml_,dEmax_,d2Emax_,LabelDataLog = self.aimd(atoms,cwd,run_dir,dft_step,gen,learnWay)
           e_siesta.append(e_aimd[0])                                   # get siesta results              
                                                                        # start training  
           trajs_ = prep_data(label=self.label,direcs=data_dir,
                              split_batch=self.batch,max_batch=self.max_batch,
-                             frame=1000,dft='siesta')                  # get trajs for training
+                             frame=1000,dft=self.dft)                  # get trajs for training
           with open('training.log','w') as fe:                         # 
                fe.write('## training loss and accuries\n')             #
-          trajs_.update(self.direcs)                                   #
+          trajs_.update(self.dataset)                                  #
 
           self.load_config()
           tf.compat.v1.disable_eager_execution()                       # training untill
@@ -330,7 +334,7 @@ class LearningMachine(object):
           training = True                   
           while (accu<=self.accCriteria or loss>=self.lossCriteria) and training:        
                 self.load_config()
-                loss,accu,accMax,p,zpe,tstep = self.trainer(direcs=trajs_,
+                loss,accu,accMax,p,zpe,tstep = self.trainer(dataset=trajs_,
                                                      ffield=self.ffield,
                                                      step=self.step,writelib=self.writelib,
                                                      batch=self.batch,
@@ -359,12 +363,18 @@ class LearningMachine(object):
                 if tIter>=self.resetAcc and tstep>=self.step:
                    if accu>self.accCriteria-self.accTole and loss<self.lossCriteria+self.lossTole:
                       self.accCriteria = float(accMax)
-                      self.lossCriteria= float(loss)
+                      loss_ = float(loss)
+                      if self.lossCriteria>loss_:
+                         self.lossCriteria = loss_ 
                       self.save_config()
                       training = False
                 tIter += 1
 
-          self.lossCriteria = float(loss)
+          loss_ = float(loss)
+          if self.lossCriteria>loss_:
+             self.lossCriteria = loss_ 
+          # self.lossCriteria  = float(loss)
+          
           if self.CheckZmat:
              if not zmatopt is None:
                 optlog = self.a.get_optimal_zv(atoms,zmatopt,self.initConfig)
@@ -408,7 +418,7 @@ class LearningMachine(object):
              if self.accCriteria<self.accMax:                           # accracy adjusted dynamicly
                 self.accCriteria = min(self.accCriteria*self.accInc,self.accMax)  
           elif accu < self.accCriteria:                                 # adjust MD steps according 
-             if self.md_step>1:                                         # training results
+             if self.md_step>30:                                        # training results
                 self.md_step  = int(self.md_step/self.mdInc)            #                 
 
           if self.md_step>=self.MaxMDstep*0.5:
@@ -490,19 +500,39 @@ class LearningMachine(object):
             l.write(optlog)
          l.write(' \n')
 
-  def aimd(self,cwd,run_dir,tstep,gen,learnWay):
+  def aimd(self,atoms,cwd,run_dir,tstep,gen,learnWay):
       mkdir(run_dir)
       chdir(cwd+'/'+run_dir)
-      popen('cp ../../*.psf ./')
+      
+      if self.dft=='siesta':
+         popen('cp ../../*.psf ./')
+      elif self.dft=='qe':
+         popen('cp ../../*.UPF ./')
+
       emlmd  = []
-      ind_   = []
+      ind_   = ' '
       dEmax  = 0.0
       d2Emax = 0.0
 
       if learnWay==1:
-         images = siesta_md(label=self.label,ncpu=self.ncpu,T=self.T,dt=self.dt_aimd,us='F',tstep=tstep,
-                            gen=cwd+'/'+gen,FreeAtoms=self.freeatoms,
-                            xcf=self.xcf,xca=self.xca,basistype=self.basistype)
+         if self.dft=='siesta':
+            # images = siesta_md(label=self.label,ncpu=self.ncpu,T=self.T,dt=self.dt_aimd,us='F',tstep=tstep,
+            #                    gen=cwd+'/'+gen,FreeAtoms=self.freeatoms,
+            #                    xcf=self.xcf,xca=self.xca,basistype=self.basistype)
+            images = siesta_opt(atoms=atoms,label=self.label,ncpu=self.ncpu,VariableCell=False,us='F',
+                                tstep=tstep,gen=cwd+'/'+gen,
+                                xcf=self.xcf,xca=self.xca,basistype=self.basistype)
+            ind_   = 'Molecular Dynamics Simulation'
+         elif self.dft=='qe':
+            # images = qemd(label=self.label,ncpu=self.ncpu,T=self.T,dt=self.dt_aimd,tstep=tstep,
+            #               gen=cwd+'/'+gen,
+            #               kpts=self.kpts)         ## geomentry optimize
+            images = qeopt(atoms=atoms,label=self.label,ncpu=self.ncpu,
+                           gen=cwd+'/'+gen,
+                           kpts=self.kpts)         ## geomentry optimize
+            ind_   = 'Geomentry Optimization'
+         else:
+            raise RuntimeError('-  This method not implimented!')
          eaimd = [images[0].get_potential_energy()]
          popen('cp %s ../../md.traj' %(self.label+'.traj'))
       elif learnWay>=2:
@@ -512,6 +542,7 @@ class LearningMachine(object):
          E,E_,dEmax,d2Emax,ind_ = SinglePointEnergies('md.traj',label=self.label,EngTole=self.EngTole,
                                                  frame=tstep,select=True,
                                                  dE=self.dEtole, # d2E=self.d2Etole,
+                                                 dft=self.dft,kpts=self.kpts,
                                                  xcf=self.xcf,xca=self.xca,basistype=self.basistype,
                                                  cpu=self.ncpu)
          eaimd = E
@@ -572,6 +603,32 @@ class LearningMachine(object):
       else:
          self.fixatoms  = FixAtoms(indices=[atom.index for atom in atoms if atom.index not in self.freeatoms])
 
+  def check_atoms(self,atoms):
+      ''' check wheter the current the speciece and cell structure 
+          consistant with initial atom structure
+      '''
+      consistance = True
+      if len(atoms)==len(self.a.atoms):
+         for i,atom in enumerate(atoms):
+             atom_ = self.a.atoms[i]
+             if atom.symbol != atom_.symbol:
+                consistance = False
+      else:
+         consistance = False
+      
+      cell  = atoms.get_cell()
+      array = cell.array
+      cell_ = self.a.atoms.get_cell()
+      array_= cell_.array
+
+      for i,a in enumerate(array):
+          a_ = array_[i]
+          for j,x in enumerate(a):
+              x_ = a_[j]
+              # print(x,x_)
+              if abs(x-x_)>0.00001:
+                 consistance = False
+      return consistance
 
   def save_config(self):
       with open('options.json','w') as fj:
@@ -666,38 +723,37 @@ class LearningMachine(object):
 
 
 if __name__ == '__main__':
-   direcs = {'nm6_5':'/home/feng/siesta/nm6_5',
-             'nm6_14':'/home/feng/siesta/nm6_14' }
+   dataset = {'nm6_5':'/home/feng/siesta/nm6_5',
+              'nm6_14':'/home/feng/siesta/nm6_14' }
 
-   lm = LearningMachine(initConfig='noco6.gen',
-                        direcs=direcs,
-                        ncpu=4,
-                        maxiter=100,
-                        dt_mlmd=0.2,dt_aimd=1.0,dft_step=5,
-                        batch=50,max_batch=50,
-                        step=20000,md_step=50,mom_step=20,
-                        colFrame=25,
-                        MaxMDstep=100,
-                        T=800,Tmax=800,
-                        convergence=0.04,accCriteria=0.9,accMax=0.962,
-                        accTole=0.01,accInc=1.001,
-                        lossCriteria=26.0,lossTole=2.0,
-                        CheckZmat=False,
-                        resetAcc=1,
-                        mdInc=1.1,
-                        nn=True,
-                        messages=1,
-                        mefac=0.005,
-                        learnWay=5,
-                        weight={'noco':2.0,'others':2.0},
-                        EngTole=0.085,dEtole=0.14,dEstop=1.0,
-                        rmin=0.7,rmax=1.3,angmax=30.0,
-                        bo_layer=[4,1],mf_layer=[6,2],be_layer=[3,1],
-                        EnergyFunction=3,MessageFunction=1,
-                        FreeAtoms=[2,3,12,13],
-                        writelib=5000,
-                        xcf='GGA',xca='PBE',basistype='split')
-   
+   lm = LearningMachine(initConfig='c14.gen',
+                     dataset=dataset,ncpu=8,
+                     maxiter=1,
+                     step=50000,md_step=100,MaxMDstep=1000,mom_step=50,colFrame=100,
+                     T=500,
+                     angmax=20.0,
+                     lossCriteria=20.0,lossTole=30.0,
+                     CheckZmat=False,
+                     regularize=True,
+                     lambda_reg=0.001,lambda_bd=1000.0,lambda_me=0.001,lambda_ang=0.01,
+                     learnWay=1,dft_step=50,
+                     beta=0.77,# FirstAtom=4,FreeAtoms=[4,5,8], 
+                     EngTole=0.01,dEtole=0.05,dEstop=2.0,
+                     EnergyFunction=1,
+                     bo_layer=[4,1],mf_layer=[9,2],be_layer=[6,1],vdw_layer=[6,1],
+                     bo_univeral_nn=None,#['O-O'],
+                     be_univeral_nn=None,#['O-O'],
+                     mf_univeral_nn=None,#['C'],
+                     spv_be=False,bore={'others':0.22},
+                     spv_ang=False,
+                     vdw_univeral_nn=None,vdwnn=True,
+                     weight={# 'no2-l':20.0,
+                             'others':2.0},
+                     optword='nocoul-nolone-nounder-noover', 
+                     writelib=1000,convergence=0.04,
+                     # xcf='GGA',xca='PBE',basistype='split'
+                     dft='qe')
+ 
    lm.run()
    lm.close()
 
