@@ -1,4 +1,5 @@
-from .siesta import single_point
+from .siesta import single_point as single_point_siesta
+from .qe import single_point as single_point_qe
 from ase.io import read,write
 from ase.io.trajectory import TrajectoryWriter,Trajectory
 from ase.calculators.singlepoint import SinglePointCalculator
@@ -8,7 +9,7 @@ import numpy as np
 
 
 def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
-                        EngTole=0.0000001,frame=50,cpu=4,
+                        EngTole=0.0000001,frame=50,cpu=4,dft='siesta',kpts=(1,1,1),
                         dE=0.2,d2E=0.1,select=False):
     ''' get single point energy and labeling data '''
     images = Trajectory(traj)
@@ -76,7 +77,12 @@ def SinglePointEnergies(traj,label='aimd',xcf='VDW',xca='DRSLL',basistype='DZP',
         dE_   = dEs[i]
         d2E_  = d2Es[i]
 
-        atoms_= single_point(atoms,xcf=xcf,xca=xca,basistype=basistype,cpu=cpu)
+        if dft=='siesta':
+           atoms_= single_point_siesta(atoms,xcf=xcf,xca=xca,basistype=basistype,cpu=cpu)
+        elif dft=='qe':
+           atoms_= single_point_qe(atoms,kpts=kpts,cpu=cpu)
+        else:
+           raise RuntimeError('-  This method not implimented!')
         e     = atoms_.get_potential_energy()
         E.append(e)
         E_.append(e_)
