@@ -134,14 +134,18 @@ def single_point(atoms,xcf='VDW',xca='DRSLL',basistype='DZP',
     return atoms
 
 
-def siesta_opt(atoms,label='siesta',ncpu=4,supercell=[1,1,1],
+def siesta_opt(atoms=None,label='siesta',ncpu=4,supercell=[1,1,1],
                VariableCell='true',xcf='VDW',xca='DRSLL',basistype='DZP',
-               dt=0.5,T=None,tstep=5000,
+               dt=0.5,T=None,tstep=200,
                us='F',P=0.0,
-               coord='cart'):
-    write_siesta_in(atoms,coord=coord,
+               gen='poscar.gen'):
+    if exists('siesta.MDE') or exists('siesta.MD_CAR'):
+       system('rm siesta.MDE siesta.MD_CAR')
+    if atoms is None:
+       atoms = read(gen)
+    write_siesta_in(atoms,coord='cart',
                     VariableCell=VariableCell,
-                    md=False,opt='CG',P=P,
+                    md=False,opt='CG',P=P,tstep=tstep,
                     xcf=xcf,xca=xca,basistype=basistype,
                     us=us) # for MD
     run_siesta(ncpu=ncpu)
@@ -172,8 +176,8 @@ def run_siesta(ncpu=12):
 
 def write_siesta_in(Atoms,coord='zmatrix',
                     VariableCell='false',
-                   fac=1.0,
-                   opt='CG',  # 'FIRE' 'SinglePoint'
+                    fac=1.0,
+                    opt='CG',  # 'FIRE' 'SinglePoint'
                     md=False,
                     xcf='VDW',
                     xca='DRSLL',
@@ -277,7 +281,7 @@ def write_siesta_in(Atoms,coord='zmatrix',
              print('MD.TargetPressure   %f GPa' %T,file=fdf)
              print('MD.ParrinelloRahmanMass 100 Ry*fs**2',file=fdf)
        else:
-          print('MD.NumCGsteps    2000 ',file=fdf)
+          print('MD.Steps         %d' %tstep,file=fdf)
           print('MD.MaxCGDispl    0.1 Ang ',file=fdf)
        print('MD.MaxForceTol   0.03 eV/Ang ',file=fdf)
        print('MD.UseSaveXV     %s ' %us,file=fdf) # T for restart
@@ -730,16 +734,18 @@ def get_siesta_cart(label='siesta'):
     for i,line in enumerate(lines):
         l = line.split()
         if len(l)>0:
-           if l[0] == 'NumberOfSpecies':
+           if l[0] == 'NumberOfSpecies': ## ths
               ns = int(l[1])
            if l[0] == 'NumberOfAtoms':
               natom = int(l[1])
+
            if l[0]=='%block':
               if l[1]=='ChemicalSpeciesLabel':
                  spl = i+1
               if l[1]=='AtomicCoordinatesAndAtomicSpecies':
                  atml= i+1
 
+
     sp = []
     for isp in range(ns):
         l = lines[spl+isp].split() 
@@ -797,14 +803,60 @@ def get_siesta_cart(label='siesta'):
     his.close()
 
 
+def get_siesta_fdf(label='siesta'):
+    fin = open(label+'.fdf','r') 
+    lines= fin.readlines()
+    fin.close()           # getting informations from input file
+    atom_name = []
 
-if __name__ == '__main__':
-   from emdk import get_structure,emdk
-   from ase.io import read,write
-   from ase import Atoms
-   from siesta import write_siesta_in
+    natom = 0
+    ns    = 0
 
-   supercell = [1,1,1]
-   A = structure(struc='cl20')*supercell
-   write_siesta_in(A,coord='zmatrix',fac=1.20)
+    for i,line in enumerate(lines):
+        l = line.split()
+        if len(l)>0:
+           if l[0] == 'NumberOfSpecies':
+              ns = int(l[1])
+           if l[0] == 'NumberOfAtoms':
+              natom = int(l[1])
+        if ns and natom:
+           break
+
+
+    spec      = []
+    positions = []
+    atom_name = []
+    cell      = []
+
+    for i,line in enumerate(lines):
+        l = line.split()
+        if len(l)>0:
+           if l[0]=='%block':
+              if l[1]=='LatticeVectors':
+                 for s in range(3):
+                     l_ = lines[i+s+1].split()
+                     cell.append([float(l_[0]),float(l_[1]),float(l_[2])])
+              if l[1]=='ChemicalSpeciesLabel' or l[1]=='ChemicalSpecieslabel':
+                 for s in range(ns):
+                     l_ = lines[i+s+1].split()
+                     spec.append(l_[2].split('.')[0])
+              if l[1]=='AtomicCoordinatesAndAtomicSpecies':
+                 for p in range(natom):
+                     l_ = lines[i+p+1].split()
+                     positions.append([float(l_[0]),float(l_[1]),float(l_[2])])
+                     # print(spec,int(l_[3])-1)
+                     atom_name.append(spec[int(l_[3])-1])
+
+    A = Atoms(atom_name,positions,cell=cell,pbc=[True,True,True])
+    A.write(label+'.gen')
+    
+# if __name__ == '__main__':
+#    from emdk import get_structure,emdk
+#    from ase.io import read,write
+#    from ase import Atoms
+#    from siesta import write_siesta_in
+
+#    supercell = [1,1,1]
+#    A = structure(struc='cl20')*supercell
+#    write_siesta_in(A,coord='zmatrix',fac=1.20)
 
