@@ -1,7 +1,6 @@
 from __future__ import print_function
 
 
-
 def init_chk_bonds(p_,pns,bonds):
     for pn in pns:
         for bd in bonds:
@@ -84,9 +83,9 @@ class Init_Check(object):
       unit = 4.3364432032e-2
       for key in p:
           k = key.split('_')[0]
-          if key == 'boc1':
-             if p[key]>50.0:
-                p[key] = value(40.0,key)
+          # if key == 'boc1':
+          #    if p[key]>50.0:
+          #       p[key] = value(50.0,key)
           if key == 'boc2':
              if p[key]<=0.05:
                 p[key]= value(1.0,key)
@@ -94,7 +93,7 @@ class Init_Check(object):
              if p[key]<=0.001:
                 p[key]= value(0.001,key)
              if p[key]>=50.0:
-                p[key]= value(50.0,key)  
+                p[key]= value(1.0,key)  
 
           if key == 'lp1':
              if p[key]>=28.0:
@@ -124,24 +123,18 @@ class Init_Check(object):
              ang= pr.split('-')
              if  ang[1]=='H':
                 p[key] = value(0.0,key)
-             else:
-                if p[key]<=0.001:
-                   p[key] = value(1.0,key)
+             # else:
+             #    if p[key]<=0.001:
+             #       p[key] = value(1.0,key)
 
           if k=='val2': 
              if p[key]>=8.0:
                 p[key] = value(1.0,key)
-             if p[key]<=0.001:
-                p[key] = value(0.01,key)
           if k=='val3': 
-             # if p[key]<=0.001:
-             #    p[key] = value(1.0,key)
              if p[key]>=12.0:
                 p[key] = value(10.0,key)
 
           if k=='val4': 
-             # if p[key]<=0.001:
-             #    p[key] = value(1.0,key)
              if p[key]>=10.0:
                 p[key] = value(5.0,key)
           if k=='val7': 
@@ -159,13 +152,13 @@ class Init_Check(object):
           if k=='val9': 
              if p[key]>=8.0:
                 p[key] = value(8.0,key)
-             # if p[key]<=0.1:
-             #    p[key] = value(1.0,key)
+             if p[key]<=0.1:
+                p[key] = value(1.0,key)
           if k=='val10': 
              if p[key]>=15.0:
                 p[key] = value(15.0,key)
-             if p[key]<=0.1:
-                p[key] = value(1.0,key)  
+             # if p[key]<=0.1:
+             #    p[key] = value(1.0,key)  
 
           if k in ['V1','V2','V3']: 
              pr = key.split('_')[1]
@@ -190,8 +183,8 @@ class Init_Check(object):
              if p[key]>60.0:
                 p[key] = value(0.0,key)
           if key in ['pen3','pen4']:
-             if p[key]==0.0:
-                p[key] = value(1.0,key)
+             # if p[key]==0.0:
+             #    p[key] = value(1.0,key)
              if p[key]>16.0:
                 p[key] = value(8.0,key)
 
@@ -202,9 +195,9 @@ class Init_Check(object):
                 p[key] = value(3.0,key)
           if k=='be1': 
              if p[key]<=-10.0:
-                p[key] = value(-3.00,key)
+                p[key] = value(-10.00,key)
              if p[key]>=10.0:
-                p[key] = value(3.0,key)
+                p[key] = value(10.0,key)
 
           if k=='alfa': 
              if p[key]>=20.0:
@@ -224,13 +217,13 @@ class Init_Check(object):
              if p[key]>=800.0:
                 p[key]= value(600.0,key)
           if k=='Depi': 
-             if p[key]==0.0:
-                p[key]= value(0.1,key)
+             # if p[key]==0.0:
+             #    p[key]= value(0.1,key)
              if p[key]>=600.0:
                 p[key]= value(600.0,key)
           if k=='Depp': 
-             if p[key]==0.0:
-                p[key]= value(0.1,key)
+             # if p[key]==0.0:
+             #    p[key]= value(0.1,key)
              if p[key]>=600.0:
                 p[key]= value(600.0,key)
           if k=='Devdw': 
@@ -271,8 +264,8 @@ class Init_Check(object):
              if p[key]<=-16.0:
                 p[key]= value(-12.0,key)
           if k=='ovun4': 
-             if p[key]<=0.001:
-                p[key]= value(1.0,key)
+             # if p[key]<=0.001:
+             #    p[key]= value(1.0,key)
              if p[key]>=20.00:
                 p[key]= value(2.0,key)
           # if k=='ovun5': 
