@@ -28,7 +28,7 @@ def dsigmoid(x):
 
 
 class MPNN(ReaxFF):
-  def __init__(self,libfile='ffield',direcs={},
+  def __init__(self,libfile='ffield',dataset={},
                dft='ase',atoms=None,
                cons=['val','vale',
                      # 'ovun1','ovun2','ovun3','ovun4',
@@ -38,7 +38,7 @@ class MPNN(ReaxFF):
                      'coa1','coa2','coa3','coa4',
                      'pen1','pen2','pen3','pen4',
                      'Depi','Depp','cutoff','hbtol',
-                     #'val8','val9','val10','acut',
+                     'acut',#'val8','val9','val10',
                      ], # 
                nn=True,
                optmol=True,lambda_me=0.1,
@@ -63,7 +63,7 @@ class MPNN(ReaxFF):
                resetDeadNeuron=False,
                messages=1,
                TwoBodyOnly=False,
-               be_univeral_nn=None,bo_layer=[4,1],
+               be_univeral_nn=None,bo_layer=[6,1],
                bo_univeral_nn=None,be_layer=[6,1],
                mf_univeral_nn=None,mf_layer=[9,2],
                vdw_univeral_nn=None,vdw_layer=None,#[6,1],
@@ -122,7 +122,7 @@ class MPNN(ReaxFF):
       self.bdopt    = bdopt
       self.mfopt    = mfopt
 
-      ReaxFF.__init__(self,libfile=libfile,direcs=direcs,
+      ReaxFF.__init__(self,libfile=libfile,dataset=dataset,
                       dft=dft,atoms=atoms,cons=cons,opt=opt,optword=optword,
                       VariablesToOpt=VariablesToOpt,optmol=optmol,lambda_me=lambda_me,
                       nanv=nanv,batch_size=batch_size,sample=sample,
@@ -144,6 +144,7 @@ class MPNN(ReaxFF):
       self.esi      = {}
       self.fbo      = {}
 
+
   def supervise(self):
       ''' supervised learning term'''
       l_atol  = 0.0
@@ -198,7 +199,7 @@ class MPNN(ReaxFF):
                 self.diffa[bd] = tf.reduce_sum(input_tensor=tf.nn.relu((self.bo0[bd] - bom_)*fao_))
                 l_atol = tf.add(self.diffa[bd]*self.lambda_bd,l_atol)
 
-             # bop should be zero if r>rcut
+             # bop_nn = nn(bop) MUST BE zero if r>rc_bo
              fbo = tf.where(tf.less_equal(self.rbd[bd],self.rc_bo[bd]),0.0,1.0) ##### r> rc_bo that bo < botol
              self.diffb[bd]  = tf.reduce_sum(self.bop[bd]*fbo) ##### Now the bop should be eterm1,eterm2,eterm3
              l_atol  = tf.add(self.diffb[bd]*self.lambda_bd,l_atol)
@@ -269,6 +270,7 @@ class MPNN(ReaxFF):
          l_atol = tf.add(self.lambda_reg*pen_b,l_atol)
       return l_atol
 
+
   def get_loss(self):
       self.Loss = 0.0
       for mol in self.mols:
@@ -309,6 +311,7 @@ class MPNN(ReaxFF):
          self.Loss  += self.ME*self.lambda_me
       self.accuracy  = self.accuracy/self.nmol
 
+
   def build_graph(self):
       print('-  building graph ...')
       self.accuracy   = tf.constant(0.0,name='accuracy')
@@ -328,6 +331,7 @@ class MPNN(ReaxFF):
       self.get_loss()
       print('-  end of build.')
 
+
   def get_atomic_energy(self):
       i = 0
       for sp in self.spec:
@@ -430,6 +434,7 @@ class MPNN(ReaxFF):
       out= tf.reshape(o_,[nbd,self.batch])
       return out
 
+
   def get_tap(self,r,bd):
       if self.vdwnn:
          # r_ = r/self.p['rvdw_'+bd]
@@ -441,6 +446,7 @@ class MPNN(ReaxFF):
               tf.math.divide(20.0,tf.pow(self.vdwcut,7.0))*tf.pow(r,7.0)
       return tp
 
+
   def fmessage(self,pre,bd,nbd,x,layer=5):
       ''' Dimention: (nbatch,4) input = 4
                  Wi:  (4,8) 
@@ -463,6 +469,7 @@ class MPNN(ReaxFF):
       out= tf.reshape(o_,[nbd,self.batch,3])
       return out
 
+
   def get_bondorder_uc(self,bd):
       self.frc[bd] = tf.where(tf.logical_or(tf.greater(self.rbd[bd],self.rc_bo[bd]),
                                             tf.less_equal(self.rbd[bd],0.001)),
@@ -489,6 +496,7 @@ class MPNN(ReaxFF):
       self.bop_pp[bd] = fpp_ #*self.frc[bd] #*self.eterm3[bd]
       self.bop[bd]    = tf.add(self.bop_si[bd],self.bop_pi[bd]+self.bop_pp[bd],name='BOp_'+bd)
 
+
   def get_bondorder(self,t,bd,atomi,atomj):
       Di      = tf.gather_nd(self.D[t-1],self.dilink[bd])
       Dj      = tf.gather_nd(self.D[t-1],self.djlink[bd])
@@ -521,6 +529,7 @@ class MPNN(ReaxFF):
       bo       = bosi+bopi+bopp
       return bo,bosi,bopi,bopp
 
+
   def get_bond_energy(self):
       BO = tf.zeros([1,self.batch])   # for ghost atom, the value is zero
       for bd in self.bonds:
@@ -547,6 +556,7 @@ class MPNN(ReaxFF):
           self.ebda[mol] = tf.gather_nd(EBDA,self.bdlink[mol])  
           self.ebond[mol]= tf.reduce_sum(input_tensor=self.ebda[mol],axis=0,name='bondenergy')
 
+
   def message_passing(self):
       ''' finding the final Bond－order with a message passing '''
       self.H.append(self.bop)                   # 
@@ -572,13 +582,13 @@ class MPNN(ReaxFF):
                  self.Hsi[t][bd] = bosi
                  self.Hpi[t][bd] = bopi
                  self.Hpp[t][bd] = bopp
-
                  BO = tf.concat([BO,bo],0)
       
           D      = tf.gather_nd(BO,self.dlist)  
           Delta  = tf.reduce_sum(input_tensor=D,axis=1)
           self.D.append(Delta)                  # degree matrix
 
+
   def get_final_sate(self):     
       self.Delta  = self.D[-1]
       self.bo0    = self.H[-1]                  # fetch the final state 
@@ -613,6 +623,7 @@ class MPNN(ReaxFF):
       self.FBOT   = taper(self.BO0,rmin=self.atol,rmax=2.0*self.atol) 
       self.FHB    = taper(self.BO0,rmin=self.hbtol,rmax=2.0*self.hbtol) 
 
+
   def get_ebond(self,bd):
       Di      = tf.gather_nd(self.Delta,self.dilink[bd])
       Dj      = tf.gather_nd(self.Delta,self.djlink[bd])
@@ -640,6 +651,7 @@ class MPNN(ReaxFF):
          self.ppeng[bd] = self.p['Depp_'+bd]*self.esi[bd]*self.bopp[bd]
          self.EBD[bd]   = - self.sieng[bd] - self.pieng[bd] - self.ppeng[bd] 
           
+
   def set_m(self):
       ''' set variable for neural networks '''
       self.m = {}
@@ -662,12 +674,15 @@ class MPNN(ReaxFF):
           if not self.bo_univeral_nn is None:
              self.set_univeral_wb(pref=p_,bd=self.bo_univeral_nn[0],reuse_m=reuse_m,
                                     nin=1,nout=1,layer=self.bo_layer,
-                                    nnopt=self.mpopt[0],bias=2.0)
+                                    nnopt=self.mpopt[0],bias=-1.0)
           self.set_wb(pref=p_,reuse_m=reuse_m,nin=1,nout=1,layer=self.bo_layer,
-                      vlist=self.bonds,nnopt=self.mpopt[0],bias=2.0)
+                      vlist=self.bonds,nnopt=self.mpopt[0],bias=-1.0)
  
       ############ set weight and bias for message neural network ###################
-      reuse_m = True if self.mf_layer==self.mf_layer_ else False
+      if self.mf_layer==self.mf_layer_ and  self.EnergyFunction==self.EnergyFunction_:
+         reuse_m = True  
+      else:
+         reuse_m = False
       for t in range(1,self.messages+1):
           b = 0.881373587 if t>1 else -0.867
           nout_ = 1 if self.MessageFunction==1 or self.MessageFunction==3 else 3
@@ -699,6 +714,7 @@ class MPNN(ReaxFF):
          self.set_wb(pref='fv',reuse_m=reuse_m,nin=1,nout=1,layer=self.vdw_layer,
                      vlist=self.bonds,nnopt=self.mpopt[-1],bias=-0.867)
 
+
   def set_wb(self,pref='f',reuse_m=True,nnopt=True,nin=8,nout=3,layer=[8,9],vlist=None,bias=0.0):
       ''' set matix varibles '''
       if self.m_ is None:
@@ -717,7 +733,7 @@ class MPNN(ReaxFF):
           else:
              self.m[pref+'wi_'+bd] = tf.Variable(tf.random.normal([nin,layer[0]],stddev=0.1),name=pref+'wi_'+bd)   
              self.m[pref+'bi_'+bd] = tf.Variable(tf.random.normal([layer[0]],stddev=0.1),
-                                                  name=pref+'bi_'+bd)  
+                                                 name=pref+'bi_'+bd)  
 
           self.m[pref+'w_'+bd] = []                                    # hidden layer
           self.m[pref+'b_'+bd] = []
@@ -845,6 +861,7 @@ class MPNN(ReaxFF):
              self.m[pref+'b'].append(tf.Variable(tf.random.normal([layer[0]], 
                                                  stddev=0.1),name=pref+'bh'+str(i))) 
 
+
   def get_univeral_nn(self):
       self.univeral_nn = []
       if not self.bo_univeral_nn is None:
@@ -893,6 +910,7 @@ class MPNN(ReaxFF):
              for t in range(1,self.messages+1):
                  self.univeral_nn.append('f'+str(t)+'_'+sp)
 
+
   def write_lib(self,libfile='ffield',loss=None):
       p_   = self.sess.run(self.p)
       self.p_ = {}
@@ -900,6 +918,9 @@ class MPNN(ReaxFF):
       self.MolEnergy_ = self.sess.run(self.MolEnergy)
       for key in self.MolEnergy_:
           self.MolEnergy_[key] = float(self.MolEnergy_[key])
+          
+      if not loss is None:
+         self.p_['score'] = -loss
 
       for k in p_:
           key = k.split('_')[0]
