@@ -112,7 +112,7 @@ def DIV_IF(y,x):
 
 
 class ReaxFF(object):
-  def __init__(self,libfile='ffield',direcs={},
+  def __init__(self,libfile='ffield',dataset={},
                dft='ase',atoms=None,
                cons=['val','vale','lp3','cutoff','hbtol'],# 'acut'
                opt=None,optword='nocoul',
@@ -157,10 +157,10 @@ class ReaxFF(object):
            Intelligence ReaxFF Neual Network: Evoluting the Force Field parameters on-the-fly
            2017-11-01
       '''
-      self.direcs        = direcs
+      self.dataset       = dataset 
       self.libfile       = libfile
       self.batch         = batch_size
-      self.sample        = sample
+      self.sample        = sample       # uniform or random
       self.lx            = np.linspace(0,self.batch,self.batch)
       self.opt           = opt
       self.VariablesToOpt= VariablesToOpt
@@ -214,6 +214,7 @@ class ReaxFF(object):
       self.lambda_bd     = lambda_bd
       self.logger        = logger('training.log')
       self.initialized   = False  
+      self.sess_build    = False
 
       if self.VariablesToOpt is None:
          self.set_parameters(self.opt) 
@@ -258,13 +259,13 @@ class ReaxFF(object):
       self.mols   = []
       self.eself,self.evdw_,self.ecoul_ = {},{},{}
 
-      for mol in self.direcs: 
+      for mol in self.dataset: 
           nindex = []
           for key in molecules:
-              if self.direcs[key]==self.direcs[mol]:
+              if self.dataset[key]==self.dataset[mol]:
                  nindex.extend(molecules[key].indexs) 
           data_ =  get_data(structure=mol,
-                                direc=self.direcs[mol],
+                                direc=self.dataset[mol],
                                   dft=self.dft,
                                 atoms=self.atoms,
                                vdwcut=self.vdwcut,
@@ -641,7 +642,7 @@ class ReaxFF(object):
 
           NLP_    = self.nlp[sp]  if i==0 else tf.concat((NLP_,self.nlp[sp]),0)
           DANG_   = self.dang[sp] if i==0 else tf.concat((DANG_,self.dang[sp]),0)
-          i          += 1
+          i      += 1
 
       self.Dang= tf.gather_nd(DANG_,self.dalink)   
       self.NLP = tf.gather_nd(NLP_,self.dalink) 
@@ -1304,7 +1305,7 @@ class ReaxFF(object):
           elif key=='atomic':
              self.p[k] = tf.clip_by_value(self.v[k],-99.9999,999.9999)
           elif key=='acut':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0000999,0.0002)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0000999,0.002)
           elif key=='hbtol':
              self.p[k] = tf.clip_by_value(self.v[k],0.0001,0.0002)
           elif key=='cutoff':
@@ -1381,7 +1382,7 @@ class ReaxFF(object):
           elif key == 'alfa':
              self.p[k] = tf.clip_by_value(self.v[k],0.001,20.0)
           elif key== 'lp1':
-             self.p[k] = tf.clip_by_value(self.v[k],10.0,50.0)
+             self.p[k] = tf.clip_by_value(self.v[k],1.0,50.0)
           elif key== 'lp2':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,99.0*self.unit)
           elif key== 'lp3':
@@ -1408,13 +1409,13 @@ class ReaxFF(object):
              rvdw_ = 1.3*self.p_['rosi_'+vn[1]]
              self.p[k] = tf.clip_by_value(self.v[k],rvdw_,2.5*self.p_['rosi_'+vn[1]])
           elif key=='val5':
-             self.p[k] = tf.clip_by_value(self.v[k],0.0,10.0)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0,20.0)
           elif key=='val3':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,8.0)
           elif key=='val1':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,200.0*self.unit)
           elif key=='val2':
-             self.p[k] = tf.clip_by_value(self.v[k],0.3,8.0)
+             self.p[k] = tf.clip_by_value(self.v[k],0.0,8.0)
           elif key=='val9':
              self.p[k] = tf.clip_by_value(self.v[k],0.0,8.0)
           elif key=='val':
@@ -1696,24 +1697,40 @@ class ReaxFF(object):
             optimizer = tf.compat.v1.train.MomentumOptimizer(learning_rate,0.9)  #momentum=0.9
          elif method=='RMSPropOptimizer':
             optimizer = tf.compat.v1.train.RMSPropOptimizer(learning_rate)
-         elif method=='NadagradOptimizer':
-            optimizer = tf.train.NadagradOptimizer(learning_rate) 
-
-         if self.to_train and self.optMethod=='ADAM':
+         # elif method=='NadagradOptimizer':
+         #    optimizer = tf.compat.v1.train.NadagradOptimizer(learning_rate) 
+         else:
+            raise RuntimeError('-  This method not implimented!')
+            
+         if self.to_train: # and self.optMethod=='ADAM':
             self.train_step = optimizer.minimize(self.Loss)
 
       self.sess.run(tf.compat.v1.global_variables_initializer())  
       self.sess_build = True
 
 
-  def update(self):
+  def update(self,p=None):
       self.logger.info('-  updating variables ...')
+      # if optcoul==True:
+      # else:
       upop = []
       for key in self.v:
-          if key in self.opt:
-             if not key in self.cons:
-                upop.append(tf.compat.v1.assign(self.v[key],self.p_[key]))
+          k = key.split('_')[0]
+          if k in self.opt or key in self.opt:
+             if not (key in self.cons or k in self.cons):
+                if p is None:
+                   p_ = self.p_[key]*self.unit if k in self.punit else self.p_[key]
+                   upop.append(tf.compat.v1.assign(self.v[key],p_))
+                else:
+                   if key in p:
+                      p_ = p[key]*self.unit if k in self.punit else p[key]
+                      upop.append(tf.compat.v1.assign(self.v[key],p_))
+                      # print(key,p[key])
       self.sess.run(upop)
+      # for k in self.v:
+      #     key       = k.split('_')[0]
+      #     self.p[k] = self.v[k]
+      # self.write_lib(libfile=self.libfile.split('.')[0]) 
 
 
   def reset(self,opt=[],libfile=None):
@@ -1733,39 +1750,44 @@ class ReaxFF(object):
 
 
   def run(self,learning_rate=1.0e-4,method='AdamOptimizer',
-               step=2000,print_step=10,writelib=100):
+               step=2000,print_step=10,writelib=100,close_session=True):
       if not self.initialized:
          self.initialize()
-      self.session(learning_rate=learning_rate,method=method)  
+      if not self.sess_build:
+         self.session(learning_rate=learning_rate,method=method)  
 
       accs_={}
       libfile = self.libfile.split('.')[0]
 
       for i in range(step+1):
           loss,latol,ME_,accu,accs,_ = self.sess.run([self.Loss,
-                                                  self.loss_atol,
-                                                  self.ME,
-                                                  self.accuracy,
-                                                  self.accur,
-                                                  self.train_step],
+                                                      self.loss_atol,
+                                                      self.ME,
+                                                      self.accuracy,
+                                                      self.accur,
+                                                      self.train_step],
                                                   feed_dict=self.feed_dict)
           if i==0:
              accMax = accu
           else:
              if accu>accMax:
                 accMax = accu
+
+          if np.isnan(loss):
+             if close_session:
+                self.logger.info('NAN error encountered at step %d loss is %f.' %(i,loss/self.nframe))
+                # send_msg('NAN error encountered at step %d loss is %f.' %(i,loss/self.nframe))
+                tf.compat.v1.reset_default_graph()
+                self.sess.close()
+                return 9999999.9,-999.0,accMax,i,0.0
+             else:
+                break
+
           if self.optmol:
              loss_ = loss - latol - ME_*self.lambda_me
           else:
              loss_ = loss - latol
-             
-          if np.isnan(loss):
-             self.logger.info('NAN error encountered at step %d loss is %f.' %(i,loss/self.nframe))
-             # send_msg('NAN error encountered at step %d loss is %f.' %(i,loss/self.nframe))
-             tf.compat.v1.reset_default_graph()
-             self.sess.close()
-             return 0.0,0.0,accMax,i,0.0
-             
+
           if i%print_step==0:
              current = time.time()
              elapsed_time = current - self.time
@@ -1795,13 +1817,16 @@ class ReaxFF(object):
              self.plot_result(None,E,dfte)
              self.write_lib(libfile=libfile,loss=loss_)
              print('-  Convergence Occurred, job compeleted.')
-             tf.compat.v1.reset_default_graph()
-             self.sess.close()
-             return loss_,accu,accMax,i,zpe
+             break
+      
+      self.loss_ = loss_ if not (np.isnan(loss) or np.isinf(loss)) else 9999999.9
+      if self.loss_ < 9999999.0:
+         self.write_lib(libfile=libfile,loss=loss_)
 
-      tf.compat.v1.reset_default_graph()
-      self.sess.close()
-      return loss_,accu,accMax,i,zpe
+      if close_session:
+         tf.compat.v1.reset_default_graph()
+         self.sess.close()
+         return loss_,accu,accMax,i,zpe
 
 
   def feed_data(self,indexs=None):
@@ -1896,6 +1921,8 @@ class ReaxFF(object):
   def write_lib(self,libfile='ffield',loss=None):
       p_   = self.sess.run(self.p)
       self.p_ = {}
+      if not loss is None:
+         self.p_['score'] = -loss
       
       self.MolEnergy_ = self.sess.run(self.MolEnergy)
       for key in self.MolEnergy_:
@@ -1915,7 +1942,7 @@ class ReaxFF(object):
       if self.libfile.endswith('.json'):
          # print(' * save parameters to file ...')
          fj = open(libfile+'.json','w')
-         j = {'p':self.p_,'m':self.m_,
+         j = {'p':self.p_,'m':None,
               'EnergyFunction':0,
               'MessageFunction':1, 
               'messages':1,
@@ -2068,11 +2095,11 @@ class ReaxFF(object):
 
 
 
-def test_reax(direcs=None,batch_size=200,dft='siesta'):
+def test_reax(dataset=None,batch_size=200,dft='siesta'):
     ''' test reax with GULP, and run validation-set'''
     RE,GE={},{}
     rn = ReaxFF(libfile='ffield',
-                direcs=direcs,dft=dft,
+                dataset=dataset,dft=dft,
                 opt=[],optword='nocoul',
                 batch_size=batch_size,
                 clip_op=False,
@@ -2082,7 +2109,7 @@ def test_reax(direcs=None,batch_size=200,dft='siesta'):
     rn.session(learning_rate=1.0e-10,method='AdamOptimizer')
     
     
-    for mol in direcs:
+    for mol in dataset:
 
         RE['Total-Energy'] = rn.get_value(rn.E[mol])
         RE['ebond']        = rn.get_value(rn.ebond[mol])
