@@ -1,6 +1,3 @@
-from __future__ import print_function
-# import matplotlib
-# matplotlib.use('Agg')
 import matplotlib.pyplot as plt
 from matplotlib.ticker import MultipleLocator
 import numpy as np
