@@ -19,19 +19,22 @@ def q(gen='packed.gen'):
 
 
 def i():
-    p,zpe,spec,bonds,offd,angs,torp,hbs= read_lib(libfile='ffield-FromPaperSort')
+    p,zpe,spec,bonds,offd,angs,torp,hbs= read_lib(libfile='ffield-C')
 
-    fj = open('ffield-FromPaperSort.json','w')
+    fj = open('ffield.json','w')
     # j = {'p':p,'m':[],'bo_layer':[],'zpe':[]}
     j = {'p':p,'m':None,
          'EnergyFunction':0,
          'MessageFunction':0, 
          'messages':1,
          'bo_layer':None,
-         'bf_layer':None,
+         'mf_layer':None,
          'be_layer':None,
          'vdw_layer':None,
-         'MolEnergy':None}
+         'MolEnergy':{} ,
+         'rcut':None,
+         'rEquilibrium':None,
+         'rcutBond':None}
     js.dump(j,fj,sort_keys=True,indent=2)
     fj.close()
 
@@ -121,6 +124,43 @@ def s():
     js.dump(j,fj,sort_keys=True,indent=2)
     fj.close()
 
+    # cons=['val','vale',
+    #       # 'ovun1', 'ovun2','ovun3','ovun4',
+    #       # 'ovun5','ovun6','ovun7','ovun8',
+    #       # 'lp2', # 'lp1',
+    #       'lp3',
+    #       # 'bo1','bo2','bo3','bo4','bo5','bo6',
+    #       'cot1','cot2',
+    #       'coa1','coa2','coa3','coa4',
+    #       'pen1','pen2','pen3','pen4',
+    #       #'Devdw','rvdw',# 'gammaw',
+    #       'Dehb','rohb','hb1','hb2','hbtol',
+    #       'Depi','Depp','cutoff','acut',
+    #       #'val1_O-O-O','val1_O-O-N','val1_H-C-O','val2_H-C-H','val2_H-C-N',
+    #       'val8','val9','val10', # 1.0,2.5,3.0
+    #       ]
+
+def v():
+    p,zpe,spec,bonds,offd,angs,torp,hbs= read_lib(libfile='ffield')
+
+    with open('ffield.json','r') as lf:
+         j = js.load(lf)
+
+    pv =   ['ovun1', 'ovun2','ovun3','ovun4',
+            'ovun5','ovun6','ovun7','ovun8',
+            'lp2',  'lp1', 
+            'gammaw','vdw1','rvdw','Devdw','alfa']
+
+
+    for k in p:
+        key = k.split('_')[0]
+        if key in pv:
+           j['p'][k] = p[k]
+
+    with open('ffield_.json','w') as fj:
+         # j = {'p':p,'m':m,'bo_layer':bo_layer,'zpe':zpe}
+         js.dump(j,fj,sort_keys=True,indent=2)
+
 
 def f(ffield='ffield.json'):
     with open(ffield,'r') as lf:
@@ -194,6 +234,6 @@ def init_bonds(p_):
 if __name__ == '__main__':
    ''' use commond like ./gmd.py nvt --T=2800 to run it'''
    parser = argparse.ArgumentParser()
-   argh.add_commands(parser, [q,i,ii,j,jj,s,ct,t])
+   argh.add_commands(parser, [q,i,ii,j,jj,s,t,v])
    argh.dispatch(parser)
 
